/******************************************************************************
* File:   50_LCD.c
* Author: T McGee                                                            
* Created: July 16, 2012                                                     
******************************************************************************/

#include "MAX32_Buss _Cal.h"

/******************************************************************************
*  Function Name:  void LCD_Init(void)                                       
*  Return Value:   void                                                      
*  Parameters:     void                                                      
*  Description:    Initialise LCD panel, and Touch Screen.
******************************************************************************/
void LCD_Init()
{
    //Init LCD screen
    SEL_RST;                        // #1 Hardware Reset
    SEL_GP;                         // Pull low to reset.
    delayMS(5);                     // Delay 10mS
    DES_RST;
    DESELECT;
    delayMS(10);                    // Delay 20mS

    SEL_CM;
    SEL_GP;                         
    comWritPort(0x00,0xE2);         // #2 Set up PLL
    DESELECT;
    SEL_DT;
    SEL_GP;                          
    comWritPort(0x00,0x1D);         // N=0x1D (29)
    comWritPort(0x00,0x02);         // M=0x02  (2)  12MHz*30/3=120MHz
    comWritPort(0x00,0x54);         // Dummy Byte
    DESELECT;          

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0xE0);         // #3 Turn on PLL
    DESELECT;
    SEL_DT;
    SEL_GP;
    comWritPort(0x00,0x01);
    DESELECT;
    delayMS(1);                     // #4 Let PLL stablize

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0xE0);         // #5 Switch Clock to PLL
    DESELECT;
    SEL_DT;
    SEL_GP;
    comWritPort(0x00,0x03);
    DESELECT;
    delayMS(1);                     // Delay 1mS

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0x01);         // #6 software reset
    DESELECT;
    delayMS(100);                   // Delay 100mS
   
    SEL_CM;
    SEL_GP;                         
    comWritPort(0x00,0xE6);         // #7 Configure Dot CLK
    DESELECT;
    SEL_DT;
    SEL_GP;                          
    comWritPort(0x00,0x03);         // C_FPR = 262143
    comWritPort(0x00,0xFF);
    comWritPort(0x00,0xFF);
    DESELECT;  
    
    SEL_CM;
    SEL_GP;                         
    comWritPort(0x00,0xB0);         // #8A LCD SPECIFICATION
    DESELECT;
    SEL_DT;
    SEL_GP;                          
    comWritPort(0x00,0x24);         // Para1 0b00100100

    comWritPort(0x00,0x00);         // Para2 0B00000000

    comWritPort(0x00,0x03);         // Para3 Set HPS 799
    comWritPort(0x00,0x1F);         // Para4

    comWritPort(0x00,0x01);         // Para5 Set VPS 479
    comWritPort(0x00,0xDF);         // Para6

    comWritPort(0x00,0x00);         // Para7 Even, odd RGB sequence
    DESELECT;          
    
    SEL_CM;
    SEL_GP;                         
    comWritPort(0x00,0xB4);         // #8B HSYNC
    DESELECT;
    SEL_DT;
    SEL_GP;                          
    comWritPort(0x00,0x03);         // Para1 Set HT 927
    comWritPort(0x00,0x9F);         // Para2

    comWritPort(0x00,0x00);         // Para3 Set HPS 88
    comWritPort(0x00,0x58);         // Para4

    comWritPort(0x00,0x2F);         // Para5 Set HPW 47

    comWritPort(0x00,0x00);         // Para6 Set LPS 0
    comWritPort(0x00,0x00);         // Para7

    comWritPort(0x00,0x00);         // Para8 Set LPSPP 0
    DESELECT;                 

    SEL_CM;
    SEL_GP;                         
    comWritPort(0x00,0xB6);         // #8C VSYNC
    DESELECT;
    SEL_DT;
    SEL_GP;                          
    comWritPort(0x00,0x02);         // Para1 Set VT	524
    comWritPort(0x00,0x0C);         // Para2

    comWritPort(0x00,0x00);         // Para3 Set VPS	32
    comWritPort(0x00,0x20);         // Para4

    comWritPort(0x00,0x02);         // Para5 Set VPW	2
    
    comWritPort(0x00,0x00);         // Para6 Set FPS	0
    comWritPort(0x00,0x00);         // Para7
    DESELECT;                  

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0xBE);         // #9 PWM configuration
    DESELECT;
    SEL_DT;
    SEL_GP;
    comWritPort(0x00,0x05);         // PWM N
    comWritPort(0x00,0xFF);         // PWM dyty cycle (50% = 0x80)
    comWritPort(0x00,0x01);         // Disable DBC
    
                                    // #10 Configur GPIO (NA)

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0x29);         // #11 display on
    DESELECT;

    setXY(0, 0, 799, 479);          // #12 Frame Buffer Configuration
    
    SEL_CM;
    SEL_GP;                         
    comWritPort(0x00,0x36);         // #13 Configure rotation
    DESELECT;
    SEL_DT;
    SEL_GP;                          
    comWritPort(0x00,0b00000001);   // 
    DESELECT;  
    
    SEL_CM;
    SEL_GP;                         
    comWritPort(0x00,0xF0);         // #14 MUC Bus Type
    DESELECT;
    SEL_DT;
    SEL_GP;                          
    comWritPort(0x00,0x03);         // 0b00000011, 16 bit, 565 formate
    DESELECT;  

    delayMS(1);                     // Delay 1mS

/**Init Touch Screen************************************************************/
    
    SEL_TS;                 //Open touch screen channel
    comSPI(0x90);           //Set state with X-axis sample command
    comSPI(0x00);           //send 16 more clock cycles
    comSPI(0x00);
    DESELECT;               //Close touch screen channel

    //Set interrups for Touch Screen
    ConfigINT3((EXT_INT_PRI_3 | EXT_INT_ENABLE | FALLING_EDGE_INT));

}

void setFont(uint8_t* font)
{
    cfont.font = font;

    cfont.x_size   = cfont.font[0];
    cfont.y_size   = cfont.font[1];
    cfont.offset   = cfont.font[2];
    cfont.numchars = cfont.font[3];
}

void setForColor(UINT8 r, UINT8 g, UINT8 b)
{
    ForColorH=((r&248)|g>>5);
    ForColorL=((g&28)<<3|b>>3);
}

void setBackColor(UINT8 r, UINT8 g, UINT8 b)
{
    BackColorH=((r&248)|g>>5);
    BackColorL=((g&28)<<3|b>>3);
}

void clrScr()
{
    long i;

    clrXY;

    SEL_DT;
    for (i=0; i<384000; i++)
    {
        SEL_GP;                       // Enable Data transfer
        comWritPort(0,0);
        DESELECT;                       // Disable transfer
    }

}

void setXY(UINT16 x1, UINT16 y1, UINT16 x2, UINT16 y2)
{
    bytelong temp;
    
    SEL_CM;
    SEL_GP;                         
    comWritPort(0x00,0x2A);     // Set column address
    DESELECT;
    temp.int0 = x1;
    SEL_DT;
    SEL_GP;                          
    comWritPort(0x00,temp.b1);
    comWritPort(0x00,temp.b0);
    temp.int0 = x2;
    comWritPort(0x00,temp.b1);
    comWritPort(0x00,temp.b0);
    DESELECT; 

    SEL_CM;
    SEL_GP;                         
    comWritPort(0x00,0x2B);     // Set page address
    DESELECT;
    temp.int0 = y1;
    SEL_DT;
    SEL_GP;                          
    comWritPort(0x00,temp.b1);
    comWritPort(0x00,temp.b0);
    temp.int0 = y2;
    comWritPort(0x00,temp.b1);
    comWritPort(0x00,temp.b0);
    DESELECT; 
    
    SEL_CM;
    SEL_GP;                         
    comWritPort(0x00,0x2C);    // Write memory start
    DESELECT;
 
}

void fillScr(UINT8 r, UINT8 g, UINT8 b)
{
    long i;
    char ch, cl;

    ch=((r&248)|g>>5);
    cl=((g&28)<<3|b>>3);
    clrXY;

    SEL_DT;                         // Enable data transfer
    for (i=0; i<384000; i++)
    {
        SEL_GP;                     // Enable transfer
        comWritPort(ch,cl);
        DESELECT;                   // Disable transfer
    }
 }

void drawCircle(int x, int y, int radius, UINT8 w)
{
    UINT8 i;
    for(i=0;i<w;i++)
    {
	int f = 1 - radius;
	int ddF_x = 1;
	int ddF_y = -2 * radius;
	int x1 = 0;
	int y1 = radius;
        
	setXY(x, y + radius, x, y + radius);
        SEL_DT;
        SEL_GP;                          // Enable data transfer
        comWritPort(ForColorH,ForColorL);
        DESELECT;                          // Disable transfer
	setXY(x, y - radius, x, y - radius);
        SEL_DT;
        SEL_GP;                          // Enable data transfer
        comWritPort(ForColorH,ForColorL);
        DESELECT;                          // Disable transfer
	setXY(x + radius, y, x + radius, y);
        SEL_DT;
        SEL_GP;                          // Enable data transfer
        comWritPort(ForColorH,ForColorL);
        DESELECT;                          // Disable transfer
	setXY(x - radius, y, x - radius, y);
        SEL_DT;
        SEL_GP;                          // Enable data transfer
        comWritPort(ForColorH,ForColorL);
        DESELECT;                          // Disable transfer

        while(x1 < y1)
	{
            if(f >= 0)
            {
		y1--;
		ddF_y += 2;
		f += ddF_y;
            }
            x1++;
            ddF_x += 2;
            f += ddF_x;
            setXY(x + x1, y + y1, x + x1, y + y1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT;                          // Disable transfer
            setXY(x - x1, y + y1, x - x1, y + y1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT;                          // Disable transfer
            setXY(x + x1, y - y1, x + x1, y - y1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT;                          // Disable transfer
            setXY(x - x1, y - y1, x - x1, y - y1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT;                          // Disable transfer
            setXY(x + y1, y + x1, x + y1, y + x1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT;                          // Disable transfer
            setXY(x - y1, y + x1, x - y1, y + x1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT;                          // Disable transfer
            setXY(x + y1, y - x1, x + y1, y - x1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT;                          // Disable transfer
            setXY(x - y1, y - x1, x - y1, y - x1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT;                          // Disable transfer
	}
        radius--;
    }
}



void printChar(UINT8 c, int x, int y)
{
    UINT8 i, ix, fIndex, ch;
    UINT16 j,ch_pt;

    y += cfont.y_size;
    fIndex = cfont.x_size/8;
    ch_pt = (c-cfont.offset)*(fIndex*cfont.y_size)+4;

    for(j=0;j<cfont.y_size;j++)
    {
        setXY(x,y-j,x+cfont.x_size-1,y-j);
        SEL_DT;
        for(ix=0;ix<fIndex;ix++)
        {
            ch=fontbyte(ch_pt++);
            for(i=8;i>0;i--)
            {
                if((ch&(1<<(i-1)))!=0) 
                {
                    SEL_GP;                          // Enable data transfer
                    comWritPort(ForColorH,ForColorL);
                    DESELECT;                          // Disable transfer
                }
                else 
                {
                    SEL_GP;                          // Enable data transfer
                    comWritPort(BackColorH,BackColorL);
                    DESELECT;                        // Disable transfer
                }
            }
        }
    }
}

void printString(UINT8 *st, UINT16 x, UINT16 y)
{
    int stl, i;

    stl = strlen(st);
    for(i=0;i<stl;i++)
    {
        printChar(st[i], x, y);
        x += cfont.x_size;
    }
}

/******************************************************************************/
/* Change Notice interrupt handler is set at priority level 2,                */
/* with software context saving                                               */
/******************************************************************************/
void __ISR(_EXTERNAL_3_VECTOR, ipl3AUTO) _TSInterrupt(void)
{
    DisableINT3 ;                   //Disable external interrupt 3
    ServiceRequests.TS = 1;         //Enable flag for Touch Screen service
}

/******************************************************************************
 *  Function Name:  srTouchScreen
 *  Return Value:   void
 *  Parameters:     void
 *  Description:    Services a touch, by finding the X, Y positiion. Then calling
 *  a response.
******************************************************************************/
 void srTouchScreen(void)
 {
    char ix = TS_SAMPLES;
    bytelong temp_pt;
    unsigned long tempx = 0, tempy = 0, t;
    extern UINT8 WriteBuf[30];

    temp_pt.Long = 0;
    TP_X = 0;
    TP_Y = 0;

    t = TickGet(0);                                 //Set time 0.
    do{
        if(TS_IRQ) t = TickGet(0);                  //If port has changed, then reset core timer.
    }while((TickGet(0) - t < TS_DB_EXIT));          //Loop untill port stable for set time.

    //Open SPI with touch screen settings and get X and Y readings
    modeSPI(0, SYNC_MODE_FAST);

    SEL_TS;                                         //Open touch screen channel(Low)
    do{
        temp_pt.b0 = comSPI( 0xD0 );                //Send X-axis command. Recieve zero first time, Y-axis low byte in loop.
        if(ix != TS_SAMPLES){tempy += (temp_pt.Long >> 3);}

        temp_pt.b1 = comSPI( 0x00 );                //Send zeros for clock, recieve X-axis high byte.

        temp_pt.b0 = comSPI( 0x90 );                //Send Y-axis command, recive X-axis low byte.
        tempx += (temp_pt.Long >> 3);

        temp_pt.b1 = comSPI( 0x00 );                //Send zeros for clock, recieve Y-axis high byte.

    }while(--ix);

    temp_pt.b0 = comSPI( 0x00 );                    //Send zeros for clock, recieve last Y-axis low byte.
    tempy += (temp_pt.Long >> 3);
    DESELECT;                                       //Close touch screen channel(High)

    tempx = ((tempx >> TS_POWER)*X_Slope + X_Offset);       //Divide by power of two, average of samples
    if (tempy < 0) tempy = 0;
    TP_X = (UINT16)tempx;

    tempy = ((tempy >> TS_POWER)*Y_Slope + Y_Offset);
    if (tempy < 0) tempy = 0;
    TP_Y = (UINT16)tempy;

    sprintf(WriteBuf,"%u, %u\r", TP_X, TP_Y);       // output value to terminal
    PutUSBstring(WriteBuf);
     
    t = TickGet(0);                                 //Zero timer
    do{
       if (! TS_IRQ) t = TickGet(0);                //Wait for touch screen release
    }while((TickGet(0) - t < TS_DB_EXIT));          //Debounce touch screen then exit.

    ServiceRequests.TS = 0;
    mINT3ClearIntFlag();
    EnableINT3;
    //PutUSBarray("2\r\n", 2);
}





