/******************************************************************************
 * FileName:        PIC32_SPI.h
 * Dependencies:    GenericTypeDefs.h
 * Processor:       PIC32
 * Compiler:        CX32
 * Company:         Pacific Coast Motion
 * Author:          Thomas McGee
 * Created:         June 13, 2014, 2:49 PM
*****************************************************************************/
#ifndef SYSTEMCONFIG_H
#define	SYSTEMCONFIG_H

//#include <p32xxxx.h>
#include <plib.h>            // Include to use PIC32 peripheral libraries
#include <stdio.h>

#define GRAPHICS_SCREEN
#define TOUCH_SCREEN
#define TICK_TIMER
//#define REAL_TIME_CLOCK
//#define FLASH_MEMORY
//#define SD_CARD
//#define WIFI_CARD
#define SW1_LEDS

//Enable one and ownly one of the following three
#define TS_CALIBRATE
//#define TS_5in
//#define TS_7in

/*********************************************************************/
/******************* Pin and Register Definitions ********************/
/*********************************************************************/
#define ADDRESS_BUSS_TRIS   mPORTASetPinsDigitalOut(0x00BF)     //Address and parallel bus controls

#define GRAPHICS_BUS_TRIS   mPORTDSetPinsDigitalOut(0x00FF);\
                            mPORTESetPinsDigitalOut(0x00FF)     //Configure PortD as output for low byte (0:7)
                                                                //and PortE as output for High byte (8:15)
//Masks for Parallel and SPI communications
#define SEL_RST     mPORTAClearBits(BIT_7)              //Set ReSeT flag.
#define DES_RST     mPORTASetBits(BIT_7)                //clear ReSeT flag.
#define START_WR    mPORTAClearBits(BIT_5)              //Start Graphics Processor Write.
#define END_WR      mPORTASetBits(BIT_5)                //End Graphics Processor Write.
#define SEL_CM      mPORTAClearBits(BIT_4)              //Set Graphics Processor for Command.
#define SEL_DT      mPORTASetBits(BIT_4)                //Set Graphics Processor for Data.

#define DESELECT    mPORTASetBits(0b00001111)           //Deselect.

#define LD5_TRIS()  mPORTCSetPinsDigitalOut(BIT_1)      //Configure pin to drive LD5.
#define LD5_ON()    LATCSET = BIT_1                     //Turn on LD5.
#define LD5_OFF()   LATCCLR = BIT_1                     //Turn off LD5.
#define LD5_INV()   LATCINV = BIT_1                     //Toggle LD5. 

/*   Microchip control                CPHA spec
 * MODE    CKP     CKE          Mode    CPOL    CPHA
 *  0       0       1            0       0        0
 *  1       0       0            1       0        1
 *  2       1       1            2       1        0
 *  3       1       0            3       1        1
 */

#define SPI_MASTER          SPI_OPEN_MSTEN
#define SPI_MODE8           0x0000
#define CKP_HIGH            SPI_OPEN_CKP_HIGH
#define CKP_LOW             0x0000
#define CKE_HIGH            SPI_OPEN_CKE_REV
#define CKE_LOW             0x0000

// Registers for the SPI module you want to use
#define SPIBUF              SPI4BUF                     //Description: The SPI Buffer
#define SPISTAT_RBF         SPI4STATbits.SPIRBF         //Description: The receive buffer full bit in the SPI status register
#define SPI_MODE_STARTUP    0x00                        //SPI  mode 0
#define SPI_SPEED_STARTUP   1000000L                    //SPI module speed 1000000bits/s

/******************************************************************************
 *  User Type Definitions and Global Variables
 *****************************************************************************/
typedef union {		//Union for long, int and byte  
        long Long;
        struct {int  int0; int int1;};
        struct {char  b0; char  b1; char  b2; char  b3;};
    }bytelong;

union {
    struct{
        unsigned TS_DWN     :1;     // xxxxxxx1 Touch Screen pen down
        unsigned TS_UP      :1;     // xxxxxx1x Touch Screen pen up
        unsigned TS_DIR     :1;     // xxxxx1xx Interupt rise/fall
        unsigned SD_Card    :1;     // xxxx1xxx SDCard has changed
        unsigned WIFI       :1;     // xxx1xxxx WFI request
        unsigned EMP0       :1;     // xx1xxxxx
        unsigned EMP1       :1;     // x1xxxxxx
        unsigned EMP2       :1;     // 1xxxxxxx     
    };
    unsigned char sr_byte;           // Set all flages with byte
} ServiceRequests;

char WriteBuf[40];

#if defined(FLASH_MEMORY) || defined(SD_CARD)
char ImageLine[2500];
#endif

/******************************************************************************
 * Prototypes
 *****************************************************************************/
void Init_buss( char Mode,long bit_rate);
void modeSPI( char Mode,long bit_rate);
void comWritPort(char H_byte,char L_byte);
char comSPI( char data_out );

/*------------------------------------------------------------------------------
 Hardward dependencies
 -------------------------------------------------------------------------------*/


#ifdef TICK_TIMER
/******************************************************************************
 * Labels
 *****************************************************************************/
#define GetSystemClock()        80000000L
#define	GetPeripheralClock()    (GetSystemClock()/(1 << OSCCONbits.PBDIV))
#define TICKS_PER_SECOND        ((unsigned long) GetSystemClock()/256)      //312500tic/sec, 3.2 uS/tic
#define TICKS_PER_MILLSEC       (TICKS_PER_SECOND / 1000)                   //312.5tic/ms
#define TIMER_TICKS         0
#define TICKS_DIV_256       1
#define TICKS_DIV_65K       2

/******************************************************************************
 * Prototypes
 *****************************************************************************/
void TickInit(void);
unsigned long TickGet(char mode);
void delayCT(unsigned long Tics);
void delayMS(unsigned long milliS);
#endif

#ifdef GRAPHICS_SCREEN
/******************************************************************************
* Labels
******************************************************************************/
#define SEL_GP      mPORTAClearBits(0b00001111)         //GP address = 0bxxxx0000.

#define Main_Screen     0
#define First_Screen    1
#define Second_Screen   2
#define Third_Screen    3
#define Fourth_Screen   4

/******************************************************************************
* Global Variables
******************************************************************************/
struct              //Structure for font data
{
    char* font;
    char x_size;
    char y_size;
    char offset;
    char numchars;
}cfont;

char CurrentScreen[8];
char ScreenIndex;
char ForColorH, ForColorL, BackColorH, BackColorL;

/******************************************************************************
 * Prototypes
 *****************************************************************************/
    void LCD_Init();

    void setFont(char* font);
    void setForColor(char r, char g, char b);
    void setBackColor(char r, char g, char b);
    void clrScr();
    void setXY(int x1, int y1, int x2, int y2);

    // Graphics Functions
    void fillScr(char r, char g, char b);
    void drawRect(int x1, int y1, int x2, int y2);
    void fillRect(int x1, int y1, int x2, int y2);
    void drawRoundRect(int x1, int y1, int x2, int y2, char w);
    void fillRoundRect(int x1, int y1, int x2, int y2);
    void drawCircle(int x, int y, int radius, char w);
    void drawCircle2(int x, int y, int radius, char w);
    void draw1stQuad(int x, int y, int radius, char w);
    void draw2ndQuad(int x, int y, int radius, char w);
    void draw3rdQuad(int x, int y, int radius, char w);
    void fillCircle(int x, int y, int radius);
    void drawHLine(int x, int y, int l);
    void drawHLWide(int x, int y, int l, int w);
    void drawArrowL(int x, int y, int w);
    void drawArrowR(int x, int y, int w);
    void drawFillBar(int x, int y, int l);
    void drawVLine(int x, int y, int l);
    void drawVLWide(int x, int y, int l, int w);
    void drawLine(int x1, int y1, int x2, int y2);
    void drawPixel(int x, int y);
    void setPixel(char r, char g, char b);
    void printChar(char c, int x, int y);
    void printString(char *st, int x, int y);
    void drawBitmap(int x, int y, const unsigned char* image);
    void drawBitmapFile(int x, int y, char* image);
#endif

#ifdef TOUCH_SCREEN
/******************************************************************************
* Labels
******************************************************************************/
    
#define SEL_TS          mPORTASetBits(0b00001111);\
                        mPORTAClearBits(0b00001110)         //TS address = 0bxxxx0001

#define TS_TD_TRIS      mPORTASetPinsDigitalIn(BIT_14)       //PenIRQ TRIS bit.
#define TS_IRQ          mPORTAReadBits(BIT_14)               //PenIRQ Input bit.

#define TS_MODE         0x00            //SPI  mode 0
#define TS_SPEED        1000000L        //SPI module speed 1000000bits/s

#define TS_DB_ENTRY     6250            // Delay 10mS, (20 * 312.5 tics/milliS)
#define TS_DB_EXIT      6250            // Delay 10mS, (10 * 312.5 tics/milliS)
#define TS_POWER        3               //Power of two used.
#define TS_SAMPLES      8               //Set to(2 ^ TS_POWER)

#define ALPHA_X_CAL     1.0             //For X axis calibration
#define BETA_X_CAL      0.0
#define DELTA_X_CAL     0.0

#define ALPHA_Y_CAL     0.0             //For Y axis calibration
#define BETA_Y_CAL      1.0
#define DELTA_Y_CAL     0.0

#define ALPHA_X_5in     0.2134          //For 5in X axis
#define BETA_X_5in      0.0
#define DELTA_X_5in     -35.318

#define ALPHA_Y_5in     0.0009            //For 5in Y axis
#define BETA_Y_5in      0.1349
#define DELTA_Y_5in     -27.619

#define ALPHA_X_7in     0.2025          //For 7in X axis
#define BETA_X_7in      0.0
#define DELTA_X_7in     -11.569

#define ALPHA_Y_7in     0.0             //For 7in Y axis
#define BETA_Y_7in      0.1285
#define DELTA_Y_7in     -21.255
/******************************************************************************
* Global Variables
******************************************************************************/
unsigned int TP_X ,TP_Y;

/******************************************************************************
 * Prototypes
 *****************************************************************************/
void TS_Init(void);
void srTouchScreen(void);
void ScreenHandler(void);

//Screens used
void MainScreen(void);
void MainResponse(void);
void MainSelect(UINT16 x1, UINT16 y1, UINT16 x2, UINT16 y2, UINT8 w );
void MainDeselect(UINT16 x1, UINT16 y1, UINT16 x2, UINT16 y2, UINT8 w );
void FirstScreen(void);
void FirstResponse(void);
void FirstSelection(UINT16 x1, UINT16 y1, UINT16 x2, UINT16 y2, UINT8 w);
void SecondScreen(void);
void SecondResponse(void);
void SecondSelection(UINT16 x1, UINT16 y1, UINT16 x2, UINT16 y2, UINT8 w);
void ThirdScreen(void);
void ThirdResponse(void);
void ThirdSelection(UINT16 x1, UINT16 y1, UINT16 x2, UINT16 y2, UINT8 w);
void ForthScreen(void);
void ForthResponse(void);
void ForthSelection(UINT16 x1, UINT16 y1, UINT16 x2, UINT16 y2, UINT8 w);

#endif

#ifdef REAL_TIME_CLOCK
/******************************************************************************
 * Labels
 *****************************************************************************/
#define SEL_RTC     mPORTASetBits(0b00001111);\
                    mPORTAClearBits(0b00001100)         //Select RTCC = 0bxxxx0011 (Y3)

#define SPI_MODE_RTCC       0x01                //SPI mode 1
#define SPI_SPEED_RTCC      6000000L            //SPI speed 6Mbits/s

#define RTC_WRITE           0x20
#define RTC_READ            0xA0


/******************************************************************************
* Global Variables
******************************************************************************/
char Time[20];
/******************************************************************************
* Prototypes
*****************************************************************************/
void rtccSet2127(char sec, char min, char hour, char wday, char mday, char mon, char year);
void rtccLoadTime(void);
void RtccGetTDstring(char *buffer );
#endif

#ifdef SD_CARD
/******************************************************************************
 * Includes
 *****************************************************************************/
#include "SD_Card/SDcard_SPI.h"
#include "SD_Card/FSIO.h"

/******************************************************************************
 * Labels
 *****************************************************************************/
#define SEL_SD      mPORTASetBits(0b00001111);\
                    mPORTAClearBits(0b00001011)         //Select SD Card = 0bxxxx0100 (Y4)

#define SD_CD_TRIS  mPORTGSetPinsDigitalIn(BIT_6)       //SD-SPI Card Detect TRIS bit. CN8/RG6(J4.33) Max#50
#define SD_CD       PORTGbits.RG6                       //SD-SPI Card Detect Input bit. CN8/RG6(J4.33) Max#50

//SD Card uses SPI mode 0 or 3
#define SPI_SPEED_SDCARD_FAST       16000000L           //Sets SPI module to 1Mbits/s
#define SPI_SPEED_SDCARD_SLOW         250000L           //Sets SPI module to 0.25Mbits/s
#define SPI_MODE_SDCARD                  0x00           //SPI mode 0

/******************************************************************************
* Global Variable Declaration                                                
******************************************************************************/
FSFILE * F_Pointer;

/******************************************************************************
 * Prototypes
 *****************************************************************************/

#endif

#ifdef FLASH_MEMORY
/******************************************************************************
 * Labels
 *****************************************************************************/
#define SEL_FMEM    mPORTASetBits(0b00001111);\
                    mPORTAClearBits(0b00001101)         //Select Flash Memory = 0bxxxx0010 (Y2)

#define SPI_MODE_FLASH_MEM               0x00           //SPI mode 0
#define SPI_SPEED_FLASH_MEM         16000000L           //SPI speed 16Mbits/s

#define SPI_FLASH_SECTOR_SIZE       4096ul
#define SPI_FLASH_SECTOR_MASK       0x00000FFF          //& mask with address results in 0x00 on start of new sector

/******************************************************************************
* Global Variable Declaration
******************************************************************************/
volatile bytelong WriteAddr;        // Internal pointer to address being written
volatile bytelong ReadAddr;         // Internal pointer to address being written

/******************************************************************************
 * Prototypes
 *****************************************************************************/
void SPIFlashInit(void);
void SPIFlashReadArray(char *vData, int wLength);
//void SPIFlashWriteArray(char* vData, int wLen);   
void SPIFlashWriteWord(const unsigned char* vData, int wLen);
void SPIFlashWriteByte(const unsigned char* vData, int wLen);
void SPIFlashEraseSector(void);
#endif

#ifdef WIFI_CARD
/******************************************************************************
 * Labels
 *****************************************************************************/
#define SEL_WIFI    mPORTASetBits(0b00001111);\
                    mPORTAClearBits(0b00001010)         //Select WIFI = 0bxxxx0101 (Y5)

#define SPI_MODE_WIFI       0x00               //SPI mode 0. Uses modes 0 and 3.
#define SPI_SPEED_WIFI      1000000L           //SPI speed module to 1Mbits/s. MAX SPEED 25MHz.

#define WF_HIBERNATE_TRIS2   mPORTGSetPinsDigitalOut(BIT_12)
#define WF_HIBERNATE_IO2     PORTGbits.RG12

/******************************************************************************
* Global Variable Declaration
******************************************************************************/

/******************************************************************************
 * Prototypes
 *****************************************************************************/

#endif

#ifdef SW1_LEDS
/******************************************************************************
 * Labels
 *****************************************************************************/

// LED I/O pins
#define LEDS_TRIS()         mPORTFSetPinsDigitalOut(0x000B)

#define LED0_ON()           LATFSET = BIT_3        //Green LED
#define LED0_OFF()          LATFCLR = BIT_3
#define LED0_INV()          LATFINV = BIT_3

#define LED1_ON()           LATFSET = BIT_0        //Yellow LED
#define LED1_OFF()          LATFCLR = BIT_0
#define LED1_INV()          LATFINV = BIT_0

#define LED2_ON()           LATFSET = BIT_1        //Red LED
#define LED2_OFF()          LATFCLR = BIT_1
#define LED2_INV()          LATFINV = BIT_1

#define LEDS_ON()           {LED0_ON(); LED1_ON(); LED2_ON();}
#define LEDS_OFF()          {LED0_OFF(); LED1_OFF(); LED2_OFF();}
#define LEDS_INV()          {LED0_INV(); LED1_INV(); LED2_INV();}

//Button I/O pins
#define SW0_TRIS()          mPORTGSetPinsDigitalIn(BIT_15)
#define	SW0_IO              PORTGbits.RG15

#endif


#endif	/* SYSTEMCONFIG_H */
